#pragma once

#include <QString>

namespace datas {

	class IDataArray;
	class IDataObject {
	public:
		virtual ~IDataObject() = default;

		virtual bool isValid() const = 0;
		virtual bool isArray() const = 0;
		virtual IDataArray toArray() = 0;

		virtual bool getBool(const QString& key) const = 0;
		virtual double getDouble(const QString& key) const = 0;
		virtual int32_t getInt32(const QString& key) const = 0;
		virtual int64_t getInt64(const QString& key) const = 0;
		virtual QString getString(const QString& key) const = 0;
		virtual IDataObject getObject(const QString& key) const = 0;

		virtual void setBool(const QString& key, bool value) = 0;
		virtual void setDouble(const QString& key, double value) = 0;
		virtual void setInt32(const QString& key, int32_t value) = 0;
		virtual void setInt64(const QString& key, int64_t value) = 0;
		virtual void setString(const QString& key, const QString& value) = 0;
		virtual void setObject(const QString& key, IDataObject value) = 0;

		virtual QString toText() const = 0;
		virtual void loadFrom(const QString &text) = 0;
	};
	class IDataArray : public IDataObject {
	public:
		virtual ~IDataArray() = default;

		virtual bool isValid() const = 0;

		virtual int32_t size() const = 0;
		virtual IDataObject get(int index) const = 0;
		virtual void insert(int index, IDataObject value) = 0;
		virtual void append(IDataObject value) = 0;
	};


	template<typename T> void dataSet(IDataObject& object, const QString& key, T value);
	template<typename T> T dataGet(IDataObject& object, const QString& key);

}
